#!/usr/bin/env python
###########################################################################
#
# Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
#
# PyQt port in November 2010 by Hans-Peter Jansen <hpj@urpla.net>
#
###########################################################################

import sys

from PyQt4 import QtCore, QtGui

from Ui_dialog import Ui_Dialog

class Dialog (QtGui.QDialog):
    """ This class is a simple example of how to use QSharedMemory.
    It is a simple dialog that presents a few buttons. To compile the
    example, run make in examples/ipc. Then run the executable twice
    to create two processes running the dialog. In one of the processes,
    press the button to load an image into a shared memory segment, and
    then select an image file to load. Once the first process has loaded
    and displayed the image, in the second process, press the button to
    read the same image from shared memory. The second process displays
    the same image loaded from its new loaction in shared memory.

    The class contains a data member \l {QSharedMemory} {sharedMemory},
    which is initialized with the key "QSharedMemoryExample" to force
    all instances of Dialog to access the same shared memory segment.
    The constructor also connects the clicked() signal from each of the
    three dialog buttons to the slot function appropriate for handling
    each button.
    """
    def __init__(self, parent = None):
        super(Dialog, self).__init__(parent)
        self.sharedMemory = QtCore.QSharedMemory("QSharedMemoryExample")
        self.ui = Ui_Dialog()
        self.ui.setupUi(self)
        self.ui.loadFromFileButton.clicked.connect(self.loadFromFile)
        self.ui.loadFromSharedMemoryButton.clicked.connect(self.loadFromMemory)
        self.setWindowTitle(self.tr("SharedMemory Example"))

    def loadFromFile(self):
        """ This slot function is called when the \tt {Load Image From File...}
            button is pressed on the firs Dialog process. First, it tests
            whether the process is already connected to a shared memory segment
            and, if so, detaches from that segment. This ensures that we always
            start the example from the beginning if we run it multiple times
            with the same two Dialog processes. After detaching from an existing
            shared memory segment, the user is prompted to select an image file.
            The selected file is loaded into a QImage. The QImage is displayed
            in the Dialog and streamed into a QBuffer with a QDataStream.
            Next, it gets a new shared memory segment from the system big enough
            to hold the image data in the QBuffer, and it locks the segment to
            prevent the second Dialog process from accessing it. Then it copies
            the image from the QBuffer into the shared memory segment. Finally,
            it unlocks the shared memory segment so the second Dialog process
            can access it.
            After self function runs, the user is expected to press the \tt
            {Load Image from Shared Memory} button on the second Dialog process.
            \sa loadFromMemory()
        """

        if self.sharedMemory.isAttached():
            self.detach()
        self.ui.label.setText(self.tr("Select an image file"))
        fileName = QtGui.QFileDialog.getOpenFileName(self, None, None,
                                            self.tr("Images (*.png *.xpm *.jpg)"))
        image = QtGui.QImage()
        if not image.load(fileName):
            self.ui.label.setText(self.tr("Selected file is not an image, please select another."))
            return
        self.ui.label.setPixmap(QtGui.QPixmap.fromImage(image))

        # load into shared memory
        buf = QtCore.QBuffer()
        buf.open(QtCore.QBuffer.ReadWrite)
        out = QtCore.QDataStream(buf)
        out << image
        size = buf.size()
        if not self.sharedMemory.create(size):
            self.ui.label.setText(self.tr("Unable to create shared memory segment."))
            return
        size = min(self.sharedMemory.size(), size)
        self.sharedMemory.lock()
        # copy image data from buf into shared memory area
        self.sharedMemory.data()[:] = buf.data().data()
        self.sharedMemory.unlock()

    def loadFromMemory(self):
        """ This slot function is called in the second Dialog process, when the
            user presses the \tt {Load Image from Shared Memory} button. First,
            it attaches the process to the shared memory segment created by the
            first Dialog process. Then it locks the segment for exclusive
            access, copies the image data from the segment into a QBuffer, and
            streams the QBuffer into a QImage. Then it unlocks the shared memory
            segment, detaches from it, and finally displays the QImage in the
            Dialog.
            \sa loadFromFile()
        """

        if not self.sharedMemory.attach():
            self.ui.label.setText(self.tr("Unable to attach to shared memory segment.\n"
                                          "Load an image first."))
            return
 
        buf = QtCore.QBuffer()
        ins = QtCore.QDataStream(buf)
        image = QtGui.QImage()
        self.sharedMemory.lock()
        buf.setData(self.sharedMemory.constData(), self.sharedMemory.size())
        #buf.setData(self.sharedMemory.constData())
        buf.open(QtCore.QBuffer.ReadOnly)
        ins >> image
        self.sharedMemory.unlock()
        self.sharedMemory.detach()
        self.ui.label.setPixmap(QPixmap.fromImage(image))


    def detach(self):
        """ This private function is called by the destructor to detach the
            process from its shared memory segment. When the last process
            detaches from a shared memory segment, the system releases the
            shared memory.
        """
        if not self.sharedMemory.detach():
            self.ui.label.setText(self.tr("Unable to detach from shared memory."))

    def tr(self, s):
        return QtGui.QApplication.translate("Dialog", s, None, QtGui.QApplication.UnicodeUTF8)


